/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFException;
import com.linuxense.javadbf.DBFReader;
import java.awt.Image;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.sensor.SensorDataObject;
import nl.cloudfarming.client.sensor.SensorFileNode;
import nl.cloudfarming.client.sensor.SensorProcessor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * GreenSeeker DataObject. Tries to detect v/d Borne and Normal file processors
 * based on the number of fields in the sensor file.
 *
 * @author Arjan Loermans
 */
@Messages({
    "# Fileprocessor",
    "sensor.greenseeker.fileprocessor.borne.log.displayname=Greenseeker Borne",
    "sensor.greenseeker.fileprocessor.normal.log.displayname=Greenseeker Normal",
    "# {0} - Processorprovider",
    "sensor.greenseeker.processorprovider.filefilter.description=Greenseeker data {0}",
    "# {0} - Logerror",
    "sensor.greenseeker.fileprocessor.logerror.failed_to_create_time=Could not convert {0} to time.",
    "# {0} - fileObject",
    "GreenseekerDataObject.logerror.failed_to_read_inputstream=Could not read inputstream from {0}.",
    "# {0} - fileObject",
    "GreenseekerDataObject.logerror.failed_to_read_dbf=Could not read {0} with the DBF-reader.",})
public class GreenseekerDataObject extends SensorDataObject {

    private GreenseekerFileProcessor processor = null;
    int gsType = 0;
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.sensor.greenseeker.GreenseekerDataObject");
    public static final String MIME_TYPE = "application/x-agrosense-greenseeker";
    public static final String ICON_BASE = "nl/cloudfarming/client/sensor/greenseeker/greenseeker-file-icon.png";

    public GreenseekerDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException {
        super(pf, loader);
        registerEditor(MIME_TYPE, true);
        this.gsType = fieldCount(pf);
    }

    @Override
    protected Node createNodeDelegate() {
        return new GreenseekerNode(super.createNodeDelegate());
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.sensor.greenseeker.about");
    }

    private int fieldCount(FileObject fo) {
        int count = 0;
        if (FileUtil.toFile(fo) != null) { // Template file does not actually exist yet.
            try (InputStream inputStream = fo.getInputStream()) {
                DBFReader reader = new DBFReader(inputStream);
                count = reader.getFieldCount();
            } catch (DBFException ex) {
                LOG.warning(NbBundle.getMessage(GreenseekerDataObject.class, "GreenseekerDataObject.logerror.failed_to_read_dbf", fo.getName()));
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                LOG.warning(NbBundle.getMessage(GreenseekerDataObject.class, "GreenseekerDataObject.logerror.failed_to_read_inputstream", fo.getName()));
                Exceptions.printStackTrace(ex);
            }
        }
        return count;
    }

    @Override
    protected SensorProcessor getProcessor() {
        if (this.processor == null) {
            // Check the number of fields in the file, Borne has 20 fields, Normal has 14. Set the corresponding Processor
            switch (this.gsType) {
                case GreenseekerFileProcessorBorne.nrOfFields:
                    processor = new GreenseekerFileProcessorBorne(this);
                    break;
                case GreenseekerFileProcessorNormal.nrOfFields:
                    processor = new GreenseekerFileProcessorNormal(this);
                    break;
                default:
                    LOG.log(Level.WARNING, "Unsupported number of data-fields: {0}", gsType);
                    processor = null;
                    break;
            }
        }
        return this.processor;
    }

    /**
     * Node for the Greenseeker
     */
    private class GreenseekerNode extends SensorFileNode {

        public GreenseekerNode(Node original) {
            super(original);
        }

        @Override
        public Image getIcon() {
            return ImageUtilities.loadImage(ICON_BASE);
        }
    }
}
