/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFReader;
import com.vividsolutions.jts.geom.Coordinate;
import eu.agrosense.client.grid.Grid;
import eu.agrosense.client.grid.GridProvider;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorObservation;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorEventKey;
import nl.cloudfarming.client.sensor.SensorPalette;
import nl.cloudfarming.client.sensor.SensorProcessor;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * GreenSeeker File processor abstract class. Each inheriting GreenSeeker file
 * processor implements its own parseLine method
 *
 * @author Arjan Loermans
 */
public abstract class GreenseekerFileProcessor extends SensorProcessor {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_start");
    // Other variable declarations
    private static final AppLogger LOG = AppLogFactory.getLogger(GreenseekerFileProcessor.class);
    private final ProgressHandle progressHandle;
    private final EventProducer eventProducer = new EventProducer();
    protected GreenseekerDataObject dataObject;

    /**
     * Create a GreenseekerFileProcessor instance to process the specified
     * GreenSeeker sensor file.
     *
     * @param toProcess The GreenSeeker sensor file to be processed
     */
    public GreenseekerFileProcessor(GreenseekerDataObject toProcess) {
        assert toProcess != null;
        this.dataObject = toProcess;
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_description", dataObject.getName()));
    }

    abstract protected GreenseekerSensorData parseLine(Object[] payload) throws GreenseekerParseException;

    @Override
    public Grid process() {
        List<SensorObservation> sensorData = new ArrayList<>();
        try {
            int success = 0;
            int fail = 0;
            int progress = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), dataObject.getName());

            FileObject fileObject = dataObject.getPrimaryFile();
            try (InputStream inputStream = new FileInputStream(FileUtil.toFile(fileObject))) {
                DBFReader reader = new DBFReader(inputStream);

                Object[] dbfSensorRow;
                // Start progressbar
                this.progressHandle.start(reader.getRecordCount());


                while ((dbfSensorRow = reader.nextRecord()) != null) {

                    try {
                        sensorData.add(parseLine(dbfSensorRow));
                        success++;
                    } catch (GreenseekerParseException ex) {
                        LOG.trace(ex);
                        fail++;
                    }
                    progressHandle.progress(++progress);
                }
                LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);
                if (fail > 0) {
                    LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
                }
                eventProducer.triggerEvent(SensorEventKey.SENSOR_DATA_IMPORTED);
                progressHandle.finish();
            }
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, dataObject.getName());
            LOG.trace(ex);
        }
        assert (!sensorData.isEmpty() && sensorData.size() >= 1);

        double minx = sensorData.get(0).getLongitude();
        double maxx = sensorData.get(0).getLongitude();
        double miny = sensorData.get(0).getLatitude();
        double maxy = sensorData.get(0).getLatitude();

        for (SensorObservation sd : sensorData) {
            minx = (minx < sd.getLongitude()) ? minx : sd.getLongitude();
            miny = (miny < sd.getLatitude()) ? miny : sd.getLatitude();
            maxx = (maxx > sd.getLongitude()) ? maxx : sd.getLongitude();
            maxy = (maxy > sd.getLatitude()) ? maxy : sd.getLatitude();
        }

        GridProvider gridProvider = Lookup.getDefault().lookup(GridProvider.class);
        Grid grid = gridProvider.getTemporaryGridSection(new Coordinate(minx, miny), new Coordinate(maxx, maxy));

        for (SensorObservation sd : sensorData) {
            //TODO sample data gets casted into an int, so multiply with 1000 for example
            if (sd.getNdvi() != null) {
                grid.setRasterValue(new Coordinate(sd.getLongitude(), sd.getLatitude()), (int) (sd.getNdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
            }

            if (sd.getWdvi() != null) {
                grid.setRasterValue(new Coordinate(sd.getLongitude(), sd.getLatitude()), (int) (sd.getWdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
            }
        }
        return grid;
    }

    /**
     * Generic greenseeker method to build a Greenseeker specific metadata
     * object
     *
     * @return Greenseeker metadata object
     */
    @Override
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
        int sensorHeigth = Integer.parseInt(GreenseekerPreference.GREENSEEKER_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(GreenseekerPreference.NUMBER_OF_GREENSEEKER_SENSORS.getValue());
        String greenseekerSensorSpread = GreenseekerPreference.GREENSEEKER_SENSOR_SPREAD.getValue();
        String sensorType = GreenseekerPreference.SENSOR_TYPE.getValue();
        String spectralChannels = GreenseekerPreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, greenseekerSensorSpread, sensorType, spectralChannels);
    }

    @Override
    protected String getFileDateFormat() {
        return GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    private class EventProducer extends BaseGuiEventProducer {

        public EventProducer() {
            super(SensorEventKey.class);
        }
    }
}
